;pic.asm
;Copyright (c) 2002 Marinoni Mauro

;==============================================================================

	list p=16f877
	#include <p16f877.inc>
	#include "macros.inc"
	__CONFIG _BODEN_ON & _CP_OFF & _PWRTE_OFF & _WDT_OFF & _WRT_ENABLE_ON & _HS_OSC & _DEBUG_OFF & _CPD_OFF & _LVP_OFF
	errorlevel -302

;------------------------------------------------------------------------------
;Constants
STORE_SH	EQU	.00	;Servo Positions MSB
STORE_SL	EQU	.16	;Servo Positions LSB
STORE_LEVELB	EQU	.32	;Servo Levels PortB
STORE_LEVELD	EQU	.33	;Servo Levels PortD
STORE_PERIOD8	EQU	.34	;Period/8 Length
STORE_BAUD	EQU	.35	;RS232 Baudrate Constant
STORE_BAUDTYPE	EQU	.36	;RS232 Baudrate Type


;------------------------------------------------------------------------------
;Variables in bank0

	CBLOCK 0x20			;GPRs start at 0x20
	PwmH:10				;Servos HighTime - MSB
	PwmL:10				;Servo 0 HighTime - LSB

	OutDiffH:8			;PWM diff between PortB and PortD
	OutDiffL:8
	OutNumber			;Number of active outputs
	OutFirst			;Shorter period Port on ISR
	OutOrder			;First Output Port on ISR

	Period8				;Period/8 Length
	Index				;Pwm Index
	LevelB				;Output Levels PortB
	LevelD				;Output Levels PortD

	Buffer				;Store input data
	Buffer2				;Store input data
	BufferP				;Store input data
	IndexFS				;Parser Index
	SAdr				;Store Address
	Temp				;Temporary variable
	Temp2				;Temporary variable
	TempISR				;Temporary variable (ISR routine)

	TempB				;Temporary variables
	TempD
	TempH
	TempL
	ENDC

	CBLOCK 0xF0			;on 16F877/876, these are global GPR's
	W_temp				;Temporary registers to store CPU
	STATUS_temp			;context.  W_temp's memory location
	PCLATH_temp			;must be free on all banks
	FSR_temp

	BaudType			;Baud Rate Type
	BaudConstant			;Baud Rate Constant
	ENDC

;==============================================================================
;Reset vector code
	ORG	0x0000

Reset:
	nop				;ICD debugger compatability
	movlw	high Main		;set page bits
	movwf	PCLATH
	goto    Main			;goto user code

;==============================================================================
;Interrupt vector code
	ORG	0x0004

Interrupt:
					;Save CPU context
					;----------------
	movwf	W_temp			;copy W to W_temp register
	swapf	STATUS,W 		;swap STATUS to be saved into W
					;the swapf instruction, unlike the
					;movf, affects NO status bits, which
					;is why it must be used here.
	clrf	STATUS			;bank 0, regardless of current bank
					;Clears IRP,RP1,RP0
	movwf	STATUS_temp		;save STATUS to bank 0 STATUS_temp
	movf	PCLATH,W		;register only required if using pages
					;1, 2 and/or 3
	movwf	PCLATH_temp		;Save PCLATH to bank 0 PCLATH_temp
					;register
	movf	FSR,W			;save FSR to bank 0 FST_temp register
	movwf	FSR_temp
	clrf	PCLATH			;assume that this ISR is in page 0

					;Interrupt Service Routine
					;-------------------------
	btfsc	INTCON,T0IF		;is interrupt TMR0?
	goto	Timer0ISR		;if yes goto Timer0ISR
	btfsc	PIR1,TMR1IF		;is interrupt TMR1?
	goto	Timer1ISR		;if yes goto Timer1ISR
	goto	OtherISR		;something goes wrong

Timer0ISR:
	;bsf	PORTC,5			;DEBUG

	incf	Index,1			;increment index
	movf	Index,W			;if Index is 8 then reset Index
	sublw	0x08
	btfsc	STATUS,Z
	clrf	Index

	clrf	OutNumber		;Set PWM[Index]

	movf 	Index,W			
	call	ANDMask
	andwf	LevelB,W
	movwf	PORTB
	btfss	STATUS,Z
	incf	OutNumber,1

	movf 	Index,W
	call	ANDMask
	andwf	LevelD,W
	movwf	PORTD
	btfss	STATUS,Z
	incf	OutNumber,1

	comf	Period8,W		;set timer0 value
	movwf	TMR0
	bcf	INTCON,T0IF		;interrupt served

	movf	OutNumber,W		;if OutNumber=0 don't set Timer1
	btfsc	STATUS,Z
	goto	EndISR

	movlw	0x08			;select shorter port
	movwf	TempISR
	movf 	Index,W
	call	ANDMask
	andwf	OutFirst,W
	btfss	STATUS,Z

TempISRClear:
	clrf	TempISR

Timer1Set:
	bcf	STATUS,IRP		;set timer1 value
	movf	TempISR,W
	addlw	PwmH
	addwf	Index,W
	movwf	FSR
	comf	INDF,W
	movwf	TMR1H

	movf	TempISR,W
	addlw	PwmL
	addwf	Index,W
	movwf	FSR
	comf	INDF,W
	movwf	TMR1L

	bcf	PIR1,TMR1IF		;reset pending request
	movlw	B'00110001'		;start timer1
	movwf	T1CON

	goto	EndISR

Timer1ISR:
	bcf	T1CON,TMR1ON		;stop Timer1
	bcf	PIR1,TMR1IF		;reset pending request
	;bcf	INTCON,GIE		;???

	decf	OutNumber,1
	btfsc	STATUS,Z
	goto	Timer1Stop

	clrf	TempISR
	movf 	Index,W
	call	ANDMask
	andwf	OutFirst,W
	btfss	STATUS,Z
	incf	TempISR,1

	btfsc	TempISR,0
	clrf	PORTB
	btfss	TempISR,0
	clrf	PORTD

	bcf	STATUS,IRP		;set timer1 value
	movlw	OutDiffH
	addwf	Index,W
	movwf	FSR
	comf	INDF,W
	movwf	TMR1H

	movlw	OutDiffL
	addwf	Index,W
	movwf	FSR
	comf	INDF,W
	movwf	TMR1L

	bcf	PIR1,TMR1IF		;reset pending request
	movlw	B'00110001'		;start timer1
	movwf	T1CON

	goto	EndISR

Timer1Stop:
	clrf	PORTB			;Clear PWM
	clrf	PORTD			;Clear PWM

	goto	EndISR

OtherISR:
	;Nothing to do...
EndISR:
					;Restore CPU context
					;-------------------
	movf	FSR_temp,W		;restore FSR
	movwf	FSR			;move W into FSR
	movf	PCLATH_temp,W		;restore PCLATH
	movwf	PCLATH			;move W into PCLATH
	swapf	STATUS_temp,W		;restore STATUS, sets bank to original
	movwf	STATUS			;move W into STATUS
	swapf	W_temp,F		;swap W_temp
	swapf	W_temp,W		;swap W_temp into W

	retfie				;return from interrupt

;==============================================================================
;Main program

Main:
	call	Setup			;load initial constants
	call	Boot			;boot up the chip
Loop:
	call	SerialReceive		;get a character
	movwf	Buffer
	call	CheckBuffer
	call	SerialTransmit		;send echo
	call	Parser

	goto	Loop			;loop forever

;==============================================================================
;ANDMask

ANDMask:
	addwf PCL,F			;add offset to pc to
					;generate a computed goto
	retlw B'00000001'
	retlw B'00000010'
	retlw B'00000100'
	retlw B'00001000'
	retlw B'00010000'
	retlw B'00100000'
	retlw B'01000000'
	retlw B'10000000'

;------------------------------------------------------------------------------
;ORMask

ORMask:
	addwf PCL,F			;add offset to pc to
					;generate a computed goto
	retlw B'11111110'
	retlw B'11111101'
	retlw B'11111011'
	retlw B'11110111'
	retlw B'11101111'
	retlw B'11011111'
	retlw B'10111111'
	retlw B'01111111'

;------------------------------------------------------------------------------
;Setup

Setup:
	movlw   0x07			;Reset Index PWM
	movwf   Index

	clrf	IndexFS			;Reset Index Parser

	clrf	OutFirst		;Reset Out Port Order
	clrf	OutOrder

	btfsc	PORTC,0			;Check Reset Factory Defaults
	goto	ReloadDefaults		;  [RESET on RC0=1]

	;---RELOAD---
	movlw	STORE_LEVELB		;Read output status
	call	Load
	movwf	LevelB

	movlw	STORE_LEVELD		;Read output status
	call	Load
	movwf	LevelD

	movlw	STORE_PERIOD8		;Read Period/8
	call	Load
	movwf	Period8

	movlw	STORE_BAUD		;Read Baud Generator
	call	Load
	movwf	BaudConstant

	movlw	STORE_BAUDTYPE		;Read Baud Type
	call	Load
	movwf	BaudType

	bcf	STATUS,IRP		;Read Servo Positions
	movlw	.31
	movwf	SAdr
	addlw	PwmH
	movwf	FSR
LoopLoad:
	movf	SAdr,W
	call	Load
	movwf	INDF
	decf	FSR,1
	decfsz	SAdr,1
	goto	LoopLoad
	movf	SAdr,W
	call	Load
	movwf	INDF

	goto	SetupDiff
	;---END RELOAD---

	;---FACTORY---
ReloadDefaults:
	movlw   0x00			;Set High all output
        movwf   LevelB

	movlw   0x00			;Set High all output
        movwf   LevelD

	movlw	0x30			;Set Period/8 (20ms)
        movwf   Period8

	movlw	0x40			;Set Baud Generator (19200)
        movwf   BaudConstant

	movlw	0x01			;Set Baud Type (High)
        movwf   BaudType

	bcf	STATUS,IRP		;Read Servo Positions
	movlw	.15
	movwf	Temp
	addlw	PwmH
	movwf	FSR
LoopLoadH:
	movlw	0x03
	movwf	INDF
	decf	FSR,1
	decfsz	Temp,1
	goto	LoopLoadH
	movlw	0x03
	movwf	INDF

	movlw	.15
	movwf	Temp
	addlw	PwmL
	movwf	FSR
LoopLoadL:
	movlw	0xE8
	movwf	INDF
	decf	FSR,1
	decfsz	Temp,1
	goto	LoopLoadL
	movlw	0xE8
	movwf	INDF
	goto	SetupDiff
	;---END FACTORY---

SetupDiff:
	movlw	.7
	movwf	BufferP
LoopSDiff:
	call	DiffCalc
	call	FirstCalc
	decfsz	BufferP,1
	goto	LoopSDiff
	call	DiffCalc
	call	FirstCalc

	return

;------------------------------------------------------------------------------
;Boot

Boot:
	clrf	INTCON			;Disable interrupt

	clrf	PORTA			;set Port A pins low
	clrf	PORTB			;set Port B pins low
	clrf	PORTC			;set Port C pins low
	clrf	PORTD			;set Port D pins low
	clrf	PORTE			;set Port E pins low

	movlw	B'10000000'		;set AD Converter
	movwf	ADCON0
	bsf	STATUS,RP0		;Bank1
	movlw	B'10000000'		;Left justified '00000000'
	movwf	ADCON1

	movlw	B'00111111'
	movwf	TRISA			;set Port A as input for ADC
	clrf	TRISB			;set Port B as outputs
	movlw	B'11000111'
	movwf	TRISC			;set Port A for serial comunication
	clrf	TRISD			;set Port D as outputs
	movlw	B'00000111'
	movwf	TRISE			;set Port E as input for ADC

	movlw	B'01100010'		;set serial port transmitter
	movwf	TXSTA
	btfsc	BaudType,0		;set serial port speed
	bsf	TXSTA,BRGH
	movf	BaudConstant,W		;set serial port baudrate
	movwf	SPBRG
	bcf	STATUS,RP0		;Bank0
	movlw	B'11010000'		;set serial port receiver
	movwf	RCSTA

	movlw	B'00110000'		;set timer1
	movwf	T1CON
	movf	Period8,0		;set timer0 value
	movwf	TMR0
	bsf	STATUS,RP0		;Bank1
	bcf	OPTION_REG,T0CS		;set clock source as internal
	bcf	OPTION_REG,PSA		;set prescaler assignement
	bsf	OPTION_REG,PS2		;set prescaler rate
	bsf	OPTION_REG,PS1
	bsf	OPTION_REG,PS0
	bsf	PIE1,TMR1IE		;unmask Timer1 intrerrupt
	bcf	STATUS,RP0		;Bank0
	movlw   B'11100000'		;Enable timers interrupt
	movwf   INTCON

	return

;==============================================================================
;SerialReceive
;Wait for byte to be received in USART and return with byte in W
;Returns in bank0

SerialReceive:
	Bank0				;change from unknown bank to bank0
	btfss	PIR1,RCIF		;check if data received
	goto	$-1			;wait until new data
	movf	RCREG,W			;get received data into W

	return

;------------------------------------------------------------------------------
;SerialTransmit
;Transmit byte in W register from USART
;Returns in bank0

SerialTransmit:
	Bank0				;change from unknown bank to bank0
	btfss	PIR1,TXIF		;check that buffer is empty
	goto	$-1
	call	CalcParity
	movwf	TXREG			;transmit byte

	return

;------------------------------------------------------------------------------
;Store
;Write w byte in a EEPROM location stored in SAdr
;Returns in bank0

Store:
	bcf	INTCON,GIE			;Stop ISR
	Bank3
	btfsc	EECON1,WR
	goto	$-1
	bcf	STATUS,RP0
	movwf	EEDATA
	bcf	STATUS,RP1
	movf	SAdr,W
	bsf	STATUS,RP1
	movwf	EEADR
	bsf	STATUS,RP0
	bcf	EECON1,EEPGD
	bsf	EECON1,WREN
	movlw	0x55
	movwf	EECON2
	movlw	0xAA
	movwf	EECON2
	bsf	EECON1,WR
	bcf	EECON1,WREN
	Bank0
	bsf	INTCON,GIE

	return

;------------------------------------------------------------------------------
;Load
;Read a byte from EEPROM location stored in w and return it in W
;Returns in bank0

Load:
	Bank2
	movwf	EEADR
	bsf	STATUS,RP0
	bcf	EECON1,EEPGD
	bsf	EECON1,RD
	bcf	STATUS,RP0
	movf	EEDATA,W
	bcf	STATUS,RP1
	
	return

;==============================================================================
CheckBuffer:
	clrf	Temp2
	btfsc	Buffer,0
	incf	Temp2,1
	btfsc	Buffer,1
	incf	Temp2,1
	btfsc	Buffer,2
	incf	Temp2,1
	btfsc	Buffer,3
	incf	Temp2,1
	btfsc	Buffer,4
	incf	Temp2,1
	btfsc	Buffer,5
	incf	Temp2,1
	btfsc	Buffer,6
	incf	Temp2,1
	btfsc	Buffer,7
	incf	Temp2,1
	btfsc	RCSTA,0
	incf	Temp2,1
	
	clrw
	btfss	Temp2,0			;no receive error	
	goto	endcheck
	
	movlw	0x07			;set parser to status 7 (error)
	movwf	IndexFS
	
	movlw	0x01			;receive error
	
endcheck:
	return

;==============================================================================
CalcParity:
	movwf	Temp
	movwf	Temp2
	clrf	Temp2
	btfsc	Temp,0
	incf	Temp2,1
	btfsc	Temp,1
	incf	Temp2,1
	btfsc	Temp,2
	incf	Temp2,1
	btfsc	Temp,3
	incf	Temp2,1
	btfsc	Temp,4
	incf	Temp2,1
	btfsc	Temp,5
	incf	Temp2,1
	btfsc	Temp,6
	incf	Temp2,1
	btfsc	Temp,7
	incf	Temp2,1
	
	btfsc	Temp2,0
	goto	calcparityodd
	bsf	STATUS,RP0
	bcf	TXSTA,0
	bcf	STATUS,RP0
	goto	endcalcparity
calcparityodd:
	bsf	STATUS,RP0
	bsf	TXSTA,0
	bcf	STATUS,RP0
	
endcalcparity:
	movf	Temp,W
	return

;==============================================================================
FirstCalc:
	movf	BufferP,W
	call	ORMask
	andwf	OutFirst,1

	movf 	BufferP,W			
	call	ANDMask
	andwf	LevelB,W
	btfsc	STATUS,Z
	goto	EndFirst		;PortB[Index]=0

	movf 	BufferP,W			
	call	ANDMask
	andwf	LevelD,W
	btfsc	STATUS,Z
	goto	SetFirst		;PortD[Index]=0

	movf 	BufferP,W			
	call	ANDMask
	andwf	OutOrder,W
	btfsc	STATUS,Z
	goto	EndFirst

SetFirst:
	movf	BufferP,W
	call	ANDMask
	iorwf	OutFirst,1

EndFirst:
	return

;==============================================================================
DiffCalc:
	bcf	STATUS,IRP
	movlw	PwmH
	addwf	BufferP,W
	movwf	FSR
	movf	INDF,W
	movwf	TempB

	movlw	0x08
	addwf	FSR,1
	movf	INDF,W
	movwf	TempD

	subwf	TempB,W
	btfsc	STATUS,Z
	goto	DiffL			; BH - DH = 0
	btfss	STATUS,C
	goto	DiffHD			; BH - DH < 0

	movwf	TempH			; BH - DH > 0

	bcf	STATUS,IRP
	movlw	PwmL
	addwf	BufferP,W
	movwf	FSR
	movf	INDF,W
	movwf	TempB

	movlw	0x08
	addwf	FSR,1
	movf	INDF,W
	subwf	TempB,W
	movwf	TempL
	btfss	STATUS,C
	decf	TempH,1
	
	bcf	STATUS,IRP
	movlw	OutDiffH
	addwf	BufferP,W
	movwf	FSR
	movf	TempH,W
	movwf	INDF

	movlw	0x08
	addwf	FSR,1
	movf	TempL,W
	movwf	INDF

	movf	BufferP,W
	call	ORMask
	andwf	OutOrder,1

	return
DiffHD:
	movf	TempB,W
	subwf	TempD,W
	movwf	TempH

	bcf	STATUS,IRP
	movlw	PwmL
	addwf	BufferP,W
	movwf	FSR
	movf	INDF,W
	movwf	TempB

	movlw	0x08
	addwf	FSR,1
	movf	INDF,W
	movwf	TempD
	movf	TempB,W
	subwf	TempD,W
	movwf	TempL
	btfss	STATUS,C
	decf	TempH,1
	
	bcf	STATUS,IRP
	movlw	OutDiffH
	addwf	BufferP,W
	movwf	FSR
	movf	TempH,W
	movwf	INDF

	movlw	0x08
	addwf	FSR,1
	movf	TempL,W
	movwf	INDF

	movf	BufferP,W
	call	ANDMask
	iorwf	OutOrder,1

	return
DiffL:
	bcf	STATUS,IRP
	movlw	OutDiffH
	addwf	BufferP,W
	movwf	FSR
	clrf	INDF			; OutDiffH[BufferP] = 0	

	bcf	STATUS,IRP
	movlw	PwmL
	addwf	BufferP,W
	movwf	FSR
	movf	INDF,W
	movwf	TempB

	movlw	0x08
	addwf	FSR,1
	movf	INDF,W
	subwf	TempB,W
	movwf	TempL
	btfsc	STATUS,Z
	goto	DiffLEQ			; BL- DL = 0
	btfss	STATUS,C
	goto	DiffLLE			; BL- DL < 0

	bcf	STATUS,IRP
	movlw	OutDiffL
	addwf	BufferP,W
	movwf	FSR
	movf	TempL,W
	movwf	INDF

	movf	BufferP,W
	call	ORMask
	andwf	OutOrder,1

	return
DiffLLE:
	comf	TempL,1
	incf	TempL,1

	bcf	STATUS,IRP
	movlw	OutDiffL
	addwf	BufferP,W
	movwf	FSR
	movf	TempL,W
	movwf	INDF

	movf	BufferP,W
	call	ANDMask
	iorwf	OutOrder,1

	return
DiffLEQ:
	bcf	STATUS,IRP
	movlw	OutDiffL
	addwf	BufferP,W
	movwf	FSR
	movlw	0x01
	movwf	INDF

	movf	BufferP,W
	call	ORMask
	andwf	OutOrder,1
		
	return

;==============================================================================
;Parser

Parser:
	;Bank0				;?!?
	movf	IndexFS,W
	btfsc	STATUS,Z
	goto	Pars0			;Status 0 - Init
	movwf	Temp
	decf	Temp,1
	btfsc	STATUS,Z
	goto	Pars1			;Status 1 - Set port 1
	decf	Temp,1
	btfsc	STATUS,Z
	goto	Pars2			;Status 2 - Set port 2
	decf	Temp,1
	btfsc	STATUS,Z
	goto	Pars3			;Status 3 - Motors 
	decf	Temp,1
	btfsc	STATUS,Z
	goto	Pars4			;Status 4 - Set Level Mask
	decf	Temp,1
	btfsc	STATUS,Z
	goto	Pars5			;Status 5 - Set RS232
	decf	Temp,1
	btfsc	STATUS,Z
	goto	Pars6			;Status 6 - Set Period/8
	clrf	IndexFS			;Reset Parser Index
ParsEnd:
	return

;------------------------------------------------------------------------------
;Pars0

Pars0:					;First Byte
	btfss	Buffer,7
	goto	ParsData		;0xxx:xxxx

	movlw	0x70
	andwf	Buffer,W
	btfsc	STATUS,Z
	goto	ParsCtrl		;1000:xxxx

	goto	ParsEnd

;------------------------------------------------------------------------------
;Pars1

Pars1:					;Position MSByte
	movf	Buffer,W
	andlw	0x0F
	movwf	Buffer2

	movlw	0x02
	movwf	IndexFS

	goto	ParsEnd

;------------------------------------------------------------------------------
;Pars2

Pars2:					;Position LSByte
	bcf	STATUS,IRP
	movf	BufferP,W
	addlw	PwmH
	movwf	FSR

	movf	Buffer2,W
	movwf	INDF

	movf	BufferP,W
	addlw	PwmL
	movwf	FSR

	movf	Buffer,W
	movwf	INDF

	movlw	B'00000111'
	andwf	BufferP,1

	call	DiffCalc
	call	FirstCalc

	clrf	IndexFS

	goto	ParsEnd

;------------------------------------------------------------------------------
;Pars3

Pars3:
	movlw	0xE0
	andwf	Buffer,W
	btfsc	STATUS,Z
	goto	SetServo		;000x:xxxx

	movlw	0x20
	subwf	Buffer,W
	andlw	0xFE
	btfsc	STATUS,Z
	goto	SetServos		;0010:000x

	movlw	0x40
	subwf	Buffer,W
	andlw	0xFE
	btfsc	STATUS,Z
	goto	GetServoMask		;0100:000x

	movlw	0x50
	subwf	Buffer,W
	andlw	0xFE
	btfsc	STATUS,Z
	goto	SetServoMask		;0101:000x

	movlw	0x80
	subwf	Buffer,W
	btfsc	STATUS,Z
	goto	StoreServos		;1000:0000

Pars3End:
	clrf	IndexFS

	goto	ParsEnd

;------------------------------------------------------------------------------
;Pars4

Pars4:					;Set Level Mask
	movf	Buffer,W
	btfss	Buffer2,0		;PortB
	movwf	LevelB
	btfsc	Buffer2,0		;PortD
	movwf	LevelD

	movlw	.7
	movwf	BufferP
Loop4First:
	call	FirstCalc
	decfsz	BufferP,1
	goto	Loop4First
	call	FirstCalc

	clrf	IndexFS

	goto	ParsEnd

;------------------------------------------------------------------------------
;Pars5

Pars5:					;Set BaudRate Constant
	movf	Buffer,W
	movwf	BaudConstant

	movf	Buffer2,W
	movwf	BaudType

	;apply changes

	clrf	IndexFS

	goto	ParsEnd

;------------------------------------------------------------------------------
;Pars6

Pars6:					;Set Period/8 Constant
	movf	Buffer,W
	movwf	Period8
	clrf	IndexFS

	goto	ParsEnd

;==============================================================================
;ParsData

ParsData:
	movlw	0xF0
	andwf	Buffer,W
	btfsc	STATUS,Z
	goto	SetPort			;0000:xxxx

	movwf	Temp
	sublw	0x40
	btfsc	STATUS,Z
	goto	GetAnalog		;0100:xxxx

	movlw	0x10
	subwf	Temp,W
	btfsc	STATUS,Z
	goto	GetPort			;0001:xxxx

	movlw	0x20
	subwf	Temp,W
	btfsc	STATUS,Z
	goto	StorePort		;0010:xxxx

	goto	ParsEnd

;------------------------------------------------------------------------------
;ParsCtrl

ParsCtrl:
	movlw	0x7F
	andwf	Buffer,W
	btfsc	STATUS,Z
	goto	ParsMotor		;1000:0000

	movlw	0x84
	subwf	Buffer,W
	andlw	0xFC
	btfsc	STATUS,Z
	goto	ParsRS232		;1000:01xx

	movlw	0x88
	subwf	Buffer,W
	andlw	0xFC
	btfsc	STATUS,Z
	goto	ParsPeriod		;1000:10xx

	movlw	0x8C
	subwf	Buffer,W
	andlw	0xFC
	btfsc	STATUS,Z
	goto	ParsSwitch		;1000:11xx

	goto	ParsEnd

;------------------------------------------------------------------------------
;ParsMotor

ParsMotor:
	movlw	0x03
	movwf	IndexFS

	goto	ParsEnd

;------------------------------------------------------------------------------
;ParsRS232

ParsRS232:
	btfsc	Buffer,1
	goto	SetRS232		;1000:011x
	btfsc	Buffer,0
	goto	GetRS232		;1000:0101
	goto	StoreRS232		;1000:0100

	goto	ParsEnd

;------------------------------------------------------------------------------
;ParsPeriod

ParsPeriod:
	btfsc	Buffer,1
	goto	SetPeriod		;1000:101x
	btfsc	Buffer,0
	goto	GetPeriod		;1000:1001
	goto	StorePeriod		;1000:1000

	goto	ParsEnd

;------------------------------------------------------------------------------
;ParsSwitch

ParsSwitch:
	btfsc	Buffer,1
	goto	SetSwitch		;1000:111x
	goto	GetSwitch		;1000:110x

	goto	ParsEnd

;==============================================================================
;SetPort

SetPort:
	movlw	0x0F
	andwf	Buffer,W
	movwf	BufferP

	incf	IndexFS,1

	goto	ParsEnd

;------------------------------------------------------------------------------
;GetPort

GetPort:
	movlw	0x0F			;Get MSB
	andwf	Buffer,W
	movwf	Temp
	addlw	PwmH
	bcf	STATUS,IRP
	movwf	FSR
	movf	INDF,W

	call	SerialTransmit
	
	movf	Temp,W			;Get LSB
	addlw	PwmL
	bcf	STATUS,IRP
	movwf	FSR
	movf	INDF,W

	call	SerialTransmit
	
	goto	ParsEnd

;------------------------------------------------------------------------------
;StorePort

StorePort:
	movlw	0x0F			;Store MSB
	andwf	Buffer,W
	movwf	Temp
	addlw	STORE_SH
	movwf	SAdr

	movf	Temp,W
	addlw	PwmH
	bcf	STATUS,IRP
	movwf	FSR
	movf	INDF,W

	call	Store

	movlw	STORE_SL		;Store LSB
	addwf	Temp,W
	movwf	SAdr

	movf	Temp,W
	addlw	PwmL
	bcf	STATUS,IRP
	movwf	FSR
	movf	INDF,W

	call	Store

	goto	ParsEnd

;------------------------------------------------------------------------------
;GetAnalog

GetAnalog:
	bsf	STATUS,RP0		;Bank1
	clrf	ADCON1
	bcf	STATUS,RP0		;Bank0

	btfsc	Buffer,3
	goto	ADCStart
	
	bsf	STATUS,RP0		;Bank1
	bsf	ADCON1,7		;Right justified '10000000'
	bcf	STATUS,RP0		;Bank0

ADCStart:
	clrf	Temp
	btfsc	Buffer,0
	bsf	Temp,3
	btfsc	Buffer,1
	bsf	Temp,4
	btfsc	Buffer,2
	bsf	Temp,5
	movf	Temp,W	
	iorlw	B'10000001'
	movwf	ADCON0

	movlw	0x04
	movwf	Temp
ADCHoldLoop1:
	movlw	0xFF
	movwf	Temp2
ADCHoldLoop2:
	decfsz	Temp2,1
	goto	ADCHoldLoop2
	decfsz	Temp,1
	goto	ADCHoldLoop1

	bsf	ADCON0,GO_DONE
ADCLoop:
	btfsc	ADCON0,GO_DONE
	goto	ADCLoop

	movf	ADRESH,W		;Send ADC result
	call	SerialTransmit
	
	btfsc	Buffer,3		;Exit if 8bit mode
	goto	ADCExit
	
	movlw	0x7F
	movwf	Temp
ADCSndL1:
	decfsz	Temp,1
	goto	ADCSndL1
	
	bsf	STATUS,RP0		;Bank1
	movf	ADRESL,W
	bcf	STATUS,RP0		;Bank0
	call	SerialTransmit

ADCExit:
	movlw	0x10
	movwf	Temp
ADCSndL2:
	decfsz	Temp,1
	goto	ADCSndL2

	bcf	ADCON0,0
	
	goto	ParsEnd

;------------------------------------------------------------------------------
;SetServo

SetServo:
	btfss	Buffer,4
	goto	SetServoDown
	movlw	B'00000111'
	andwf	Buffer,W
	call	ANDMask
	btfss	Buffer,3
	iorwf	LevelB,1
	btfsc	Buffer,3
	iorwf	LevelD,1

	goto	SetServoEnd
SetServoDown:
	movlw	B'00000111'
	andwf	Buffer,W
	call	ORMask
	btfss	Buffer,3
	andwf	LevelB,1
	btfsc	Buffer,3
	andwf	LevelD,1

SetServoEnd:
	movlw	B'00000111'
	andwf	Buffer,W
	movwf	BufferP
	call	FirstCalc

	goto	Pars3End

;------------------------------------------------------------------------------
;SetServos

SetServos:
	clrw
	btfsc	Buffer,0
	movlw	0xFF
	movwf	LevelB
	movwf	LevelD

	movlw	.7
	movwf	BufferP
LoopFirst:
	call	FirstCalc
	decfsz	BufferP,1
	goto	LoopFirst
	call	FirstCalc
	goto	Pars3End

;------------------------------------------------------------------------------
;SetServoMask

SetServoMask:
	movlw	0x01
	andwf	Buffer,W
	movwf	Buffer2

	movlw	0x04
	movwf	IndexFS

	goto	ParsEnd

;------------------------------------------------------------------------------
;GetServoMask

GetServoMask:
	btfss	Buffer,0
	movf	LevelB,W
	btfsc	Buffer,0
	movf	LevelD,W
	call	SerialTransmit

	goto	Pars3End

;------------------------------------------------------------------------------
;StoreServos

StoreServos:
	movlw	STORE_LEVELB
	movwf	SAdr
	movf	LevelB,W
	call	Store

	movlw	STORE_LEVELD
	movwf	SAdr
	movf	LevelD,W
	call	Store

	goto	Pars3End

;------------------------------------------------------------------------------
;StoreRS232

StoreRS232:
	movlw	STORE_BAUD
	movwf	SAdr
	movf	BaudConstant,W
	call	Store

	movlw	STORE_BAUDTYPE
	movwf	SAdr
	movf	BaudType,W
	call	Store

	goto	ParsEnd

;------------------------------------------------------------------------------
;GetRS232

GetRS232:
	movf	BaudType,W
	call	SerialTransmit

	movf	BaudConstant,W
	call	SerialTransmit
	
	goto	ParsEnd

;------------------------------------------------------------------------------
;SetRS232

SetRS232:
	movlw	0x01
	andwf	Buffer,W
	movwf	Buffer2

	movlw	0x05
	movwf	IndexFS

	goto	ParsEnd

;------------------------------------------------------------------------------
;StorePeriod

StorePeriod:
	movlw	STORE_PERIOD8
	movwf	SAdr
	movf	Period8,W
	call	Store

	goto	ParsEnd

;------------------------------------------------------------------------------
;GetPeriod

GetPeriod:
	movf	Period8,W
	call	SerialTransmit
	
	goto	ParsEnd

;------------------------------------------------------------------------------
;SetPeriod

SetPeriod:
	movlw	0x06
	movwf	IndexFS

	goto	ParsEnd

;------------------------------------------------------------------------------
;GetSwitch

GetSwitch:
	movlw	0x27
	andwf	PORTC,W

	call	SerialTransmit
	
	goto	ParsEnd

;------------------------------------------------------------------------------
;SetSwitch

SetSwitch:
	btfss	Buffer,0
	bcf	PORTC,5

	btfsc	Buffer,0
	bsf	PORTC,5

	goto	ParsEnd

;==============================================================================
; EEPROM Data
	ORG	0x2100

	de	0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03	;Servo Pos MSB
	de	0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03, 0x03
	de	0xe8, 0xe8, 0xe8, 0xe8, 0xe8, 0xe8, 0xe8, 0xe8	;Servo Pos LSB
	de	0xe8, 0xe8, 0xe8, 0xe8, 0xe8, 0xe8, 0xe8, 0xe8
	de	0x00, 0x00					;Servo Levels
	de	0x30						;Period/8 Length
	de	0x0A						;Baudrate Const
	de	0x01						;Baudrate Type

;==============================================================================

	END

;==============================================================================
;See the PIC Microcontroller RS232 IO SPBRG Calc
;http://www.piclist.com/techref/microchip/spbrgcalc.asp

				;Fosc = 20MHz, BRGH = 1
;BAUD_CONSTANT	EQU	0x81	;Constant for baud generator for   9600 baud
;BAUD_CONSTANT	EQU	0x56	;Constant for baud generator for  14400 baud
;BAUD_CONSTANT	EQU	0x40	;Constant for baud generator for  19200 baud
;BAUD_CONSTANT	EQU	0x2A	;Constant for baud generator for  28800 baud
;BAUD_CONSTANT	EQU	0x24	;Constant for baud generator for  33600 baud
;BAUD_CONSTANT	EQU	0x20	;Constant for baud generator for  38400 baud
;BAUD_CONSTANT	EQU	0x14	;Constant for baud generator for  57600 baud
;BAUD_CONSTANT	EQU	0x0A	;Constant for baud generator for 115200 baud

